print("import: 'BTrees'")
import BTrees

print("import: 'BTrees._IFBTree'")
import BTrees._IFBTree

print("import: 'BTrees._IIBTree'")
import BTrees._IIBTree

print("import: 'BTrees._IOBTree'")
import BTrees._IOBTree

print("import: 'BTrees._LFBTree'")
import BTrees._LFBTree

print("import: 'BTrees._LLBTree'")
import BTrees._LLBTree

print("import: 'BTrees._LOBTree'")
import BTrees._LOBTree

print("import: 'BTrees._OIBTree'")
import BTrees._OIBTree

print("import: 'BTrees._OLBTree'")
import BTrees._OLBTree

print("import: 'BTrees._OOBTree'")
import BTrees._OOBTree

print("import: 'BTrees._fsBTree'")
import BTrees._fsBTree

